@@ -1,4 +1,4 @@
-# Copyright 2016-2018 Fabian Hofmann (FIAS), Jonas Hoersch (KIT, IAI) and
+# Copyright 2016-2020 Fabian Hofmann (FIAS), Jonas Hoersch (KIT, IAI) and
 # Fabian Gotzens (FZJ, IEK-STE)
 
 # This program is free software; you can redistribute it and/or
@@ -24,10 +24,10 @@ power plant databases.
 
 from __future__ import absolute_import
 
-__version__ = "0.4.5"
-__author__ = "Fabian Hofmann, Jonas Hoersch"
+__version__ = "0.4.6"
+__author__ = "Fabian Hofmann, Jonas Hoersch, Fabian Gotzens"
 __copyright__ = "Copyright 2017-2020 Frankfurt Institute for Advanced Studies"
-#The rough hierarchy of this package is
+# The rough hierarchy of this package is
 #core, utils, heuristics, cleaning, matching, collection, data
 
 # Commonly used sub-modules. Imported here to provide end-user
@@ -40,8 +40,6 @@ from . import data
 #from . import matching
 #from . import collection
 from . import plot
-from .core import get_config
+from .core import get_config, package_config
 from .collection import matched_data as powerplants
 from .accessor import PowerPlantAccessor
-
-
