@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2016-2018 Fabian Hofmann (FIAS), Jonas Hoersch (KIT, IAI) and
+# Copyright 2016-2020 Fabian Hofmann (FIAS), Jonas Hoersch (KIT, IAI) and
 # Fabian Gotzens (FZJ, IEK-STE)
 
 # This program is free software; you can redistribute it and/or
@@ -42,9 +42,9 @@ cget = pycountry.countries.get
 net_caps = get_config()['display_net_caps']
 
 
-def OPSD(rawEU=False, rawDE=False, update=False,
-         statusDE=['operating', 'reserve', 'special_case', 'shutdown_temporary'],
-         config=None):
+def OPSD(rawEU=False, rawDE=False, rawDE_withBlocks=False, update=False,
+         statusDE=['operating', 'reserve', 'special_case',
+                   'shutdown_temporary'], config=None):
     """
     Importer for the OPSD (Open Power Systems Data) database.
 
@@ -75,16 +75,33 @@ def OPSD(rawEU=False, rawDE=False, update=False,
         return opsd_EU
     if rawDE:
         return opsd_DE
+    if rawDE_withBlocks:
+        DE_blocks = (opsd_DE
+                     .loc[lambda x: ~(x['block_bnetza'].isna())]
+                     .loc[lambda x: x['block_bnetza'] != x['name_bnetza']]
+                     .assign(block=lambda x: x.block_bnetza.str.strip())
+                     .loc[lambda x: ~(x.block.isin(['-', 'entfällt']))]
+                     .assign(len_block=lambda x: x.block.apply(len)))
+        upd = (DE_blocks.loc[lambda x: (x.len_block <= 6)]
+                        .loc[lambda x: (x.block.str.slice(0, 5) != 'Block')]
+                        .assign(block=lambda x: 'Block ' + x['block']))
+        DE_blocks.update(upd)
+        DE_blocks = DE_blocks.assign(name_bnetza=lambda x:
+                                     x['name_bnetza'].str.strip() + ' '
+                                     + x['block'])
+        opsd_DE.update(DE_blocks)
+        return opsd_DE.drop('Unnamed: 0', axis=1).set_index('id')
+
     opsd_EU = (opsd_EU.rename(columns=str.title)
-                     .rename(columns={'Lat': 'lat',
-                                      'Lon': 'lon',
-                                      'Energy_Source': 'Fueltype',
-                                      'Commissioned': 'YearCommissioned',
-                                      'Eic_Code': 'eic_code'})
-                     .eval('Retrofit = YearCommissioned')
-                     .assign(projectID = lambda s: 'OEU' +
-                             pd.Series(s.index.astype(str), s.index))
-                     .reindex(columns=config['target_columns']))
+               .rename(columns={'Lat': 'lat',
+                                'Lon': 'lon',
+                                'Energy_Source': 'Fueltype',
+                                'Commissioned': 'DateIn',
+                                'Eic_Code': 'EIC'})
+               .eval('DateRetrofit = DateIn')
+               .assign(projectID=lambda s: 'OEU'
+                       + pd.Series(s.index.astype(str), s.index))
+               .reindex(columns=config['target_columns']))
 
     opsd_DE = (opsd_DE.rename(columns=str.title)
                       .rename(columns={'Lat': 'lat',
@@ -93,15 +110,16 @@ def OPSD(rawEU=False, rawDE=False, update=False,
                                        'Type': 'Set',
                                        'Country_Code': 'Country',
                                        'Capacity_Net_Bnetza': 'Capacity',
-                                       'Commissioned': 'YearCommissioned',
-                                       'Eic_Code_Plant': 'eic_code',
+                                       'Commissioned': 'DateIn',
+                                       'Shutdown': 'DateOut',
+                                       'Eic_Code_Plant': 'EIC',
                                        'Id': 'projectID'})
-                      .assign(
-                      Name = lambda d: d.Name_Bnetza.fillna(d.Name_Uba),
-                      Fueltype = lambda d:
+                      .assign(Name=lambda d:
+                              d.Name_Bnetza.fillna(d.Name_Uba),
+                              Fueltype=lambda d:
                                   d.Fueltype.fillna(d.Energy_Source_Level_1),
-                      Retrofit = lambda d:
-                                  d.Retrofit.fillna(d.YearCommissioned)))
+                              DateRetrofit=lambda d:
+                                  d.Retrofit.fillna(d.DateIn)))
     if statusDE is not None:
         opsd_DE = opsd_DE.loc[opsd_DE.Status.isin(statusDE)]
     opsd_DE = opsd_DE.reindex(columns=config['target_columns'])
@@ -125,9 +143,10 @@ def OPSD(rawEU=False, rawDE=False, update=False,
                     Fueltype=lambda df: df.Fueltype.str.title().str.strip())
             .powerplant.convert_alpha2_to_country()
             .pipe(set_column_name, 'OPSD')
-#            .pipe(correct_manually, 'OPSD', config=config)
+            # .pipe(correct_manually, 'OPSD', config=config)
             .pipe(config_filter, name='OPSD', config=config)
             .pipe(gather_set_info)
+            .pipe(clean_powerplantname)
             .pipe(clean_technology))
 
 
@@ -155,44 +174,53 @@ def GEO(raw=False, config=None):
                    'Type_of_Fuel_rng2_Secondary': 'FuelClassification2',
                    'Country': 'Country',
                    'Design_Capacity_MWe_nbr': 'Capacity',
-                   'Year_Project_Commissioned': 'YearCommissioned',
-                   'Year_rng1_yr1': 'Retrofit',
+                   'Year_Project_Commissioned': 'DateIn',
+                   'Year_rng1_yr1': 'DateRetrofit',
                    'Longitude_Start': 'lon',
                    'Latitude_Start': 'lat'}
 
-#    geo = pd.read_csv(_data_in(config['GEO']['fn']), low_memory=False,
-#                      usecols=list(rename_cols.keys()))
     geo = parse_if_not_stored('GEO', config=config, low_memory=False)
     if raw:
         return geo
+    geo = geo.rename(columns=rename_cols)
 
+    units = parse_if_not_stored('GEO_units', config=config, low_memory=False)
+
+    # map from units to plants
+    units['DateIn'] = units.Date_Commissioned_dt.str[:4].astype(float)
+    units['Effiency'] = units.Unit_Efficiency_Percent.str.replace('%', '')\
+                             .astype(float) / 100
+    units = units.groupby('GEO_Assigned_Identification_Number')\
+                 .agg({'DateIn': [min, max], 'Effiency': 'mean'})
+
+    _ = geo.projectID.map(units.DateIn['min'])
+    geo['DateIn'] = (geo.DateIn.str[:4].apply(pd.to_numeric, errors='coerce')
+                     .where(lambda x: x > 1900).fillna(_))
+
+    _ = geo.projectID.map(units.DateIn['max'])
+    geo['DateRetrofit'] = geo.DateRetrofit.astype(float).fillna(_)
+
+    _ = units.Effiency['mean']
+    geo['Effiency'] = geo.projectID.map(_)
+    return (geo.assign(projectID=lambda s: 'GEO' + s.projectID.astype(str))
+            .query("Country in @countries")
+            .replace({col: {'Gas': 'Natural Gas'} for col in
+                      {'Fueltype', 'FuelClassification1',
+                       'FuelClassification2'}})
+            .pipe(gather_fueltype_info, search_col=['FuelClassification1'])
+            .pipe(gather_technology_info, search_col=['FuelClassification1'],
+                  config=config)
+            .pipe(gather_set_info)
+            .pipe(set_column_name, 'GEO')
+            .pipe(config_filter, name='GEO', config=config)
+            .pipe(clean_powerplantname)
+            .pipe(clean_technology, generalize_hydros=True)
+            .pipe(scale_to_net_capacities,
+                  (not config['GEO']['net_capacity']))
+            .pipe(config_filter, name='GEO', config=config)
+            .pipe(correct_manually, 'GEO', config=config)
+            )
 
-    return (geo.rename(columns=rename_cols)
-              .assign(Retrofit=lambda s: s.Retrofit.astype(float),
-                      projectID=lambda s: 'GEO' + s.projectID.astype(str))
-              .assign(YearCommissioned=(lambda s: s.YearCommissioned
-                                        .str[:4]
-                                        .apply(pd.to_numeric, errors='coerce')
-                                        .where(lambda x: x > 1900)
-                                        .fillna(s.Retrofit)))
-              .assign(Retrofit=lambda s: s.Retrofit.fillna(s.YearCommissioned))
-              .query("Country in @countries")
-              .replace({col: {'Gas': 'Natural Gas'} for col in
-                        {'Fueltype', 'FuelClassification1',
-                         'FuelClassification2'}})
-             .pipe(gather_fueltype_info, search_col=['FuelClassification1'])
-             .pipe(gather_technology_info, search_col=['FuelClassification1'],
-                          config=config)
-             .pipe(gather_set_info)
-             .pipe(set_column_name, 'GEO')
-             .pipe(config_filter, name='GEO', config=config)
-             .pipe(clean_powerplantname)
-             .pipe(clean_technology, generalize_hydros=True)
-             .pipe(scale_to_net_capacities,
-                   (not config['GEO']['net_capacity']))
-             .pipe(config_filter, name='GEO', config=config)
-#             .pipe(correct_manually, 'GEO', config=config)
-             )
 
 def CARMA(raw=False, config=None):
     """
@@ -211,7 +239,8 @@ def CARMA(raw=False, config=None):
 
 #    carmadata = pd.read_csv(_datconfig['CARMA']['fn'], low_memory=False)
     carma = parse_if_not_stored('CARMA', config=config, low_memory=False)
-    if raw: return carma
+    if raw:
+        return carma
 
     return (carma
             .rename(columns={'Geoposition': 'Geoposition',
@@ -247,9 +276,10 @@ def CARMA(raw=False, config=None):
             .pipe(gather_set_info)
             .pipe(clean_technology)
             .pipe(scale_to_net_capacities, not config['CARMA']['net_capacity'])
-#            .pipe(correct_manually, 'CARMA', config=config)
+            .pipe(correct_manually, 'CARMA', config=config)
             )
 
+
 def JRC(raw=False, config=None, update=False):
     """
     Importer for the JRC Hydro-power plants database retrieves from
@@ -281,25 +311,32 @@ def JRC(raw=False, config=None, update=False):
         else:
             raise(err)
 
-
     df = parse_if_not_stored('JRC', update, config, parse_func=parse_func)
     if raw:
         return df
-    return (df.rename(columns={'id': 'projectID',
-                              'name': 'Name',
-                              'installed_capacity_MW': 'Capacity',
-                              'country_code': 'Country',
-                              'type': 'Technology',
-                              'dam_height_m': 'DamHeight_m',
-                              'volume_Mm3': 'Volume_Mm3'})
-            .eval('Duration = storage_capacity_MWh / Capacity')
-            .replace({'HDAM': 'Reservoir',
-                      'HPHS': 'Pumped Storage',
-                      'HROR': 'Run-Of-River'})
-            .drop(columns = ['pypsa_id', 'GEO', 'storage_capacity_MWh'])
-            .assign(Set='Store', Fueltype='Hydro')
-            .powerplant.convert_alpha2_to_country()
-            .pipe(config_filter))
+    df = (df.rename(columns={'id': 'projectID',
+                             'name': 'Name',
+                             'installed_capacity_MW': 'Capacity',
+                             'country_code': 'Country',
+                             'type': 'Technology',
+                             'dam_height_m': 'DamHeight_m',
+                             'volume_Mm3': 'Volume_Mm3',
+                             'storage_capacity_MWh': 'StorageCapacity_MWh'})
+          .eval('Duration = StorageCapacity_MWh / Capacity')
+          .replace(dict(Technology={'HDAM': 'Reservoir',
+                                    'HPHS': 'Pumped Storage',
+                                    'HROR': 'Run-Of-River'}))
+          .drop(columns=['pypsa_id', 'GEO'])
+          .assign(Set='Store', Fueltype='Hydro')
+          .powerplant.convert_alpha2_to_country()
+          .pipe(config_filter))
+    # TODO: Temporary section to deal with duplicate identifiers in the JRC
+    # input file. Can be removed again, once the duplicates have been removed
+    # in a new release.
+    mask = df.projectID.duplicated(keep=False)
+    df.loc[mask, 'projectID'] += (df.groupby('projectID').cumcount()
+                                    .replace({0: 'a', 1: 'b', 2: 'c', 3: 'd'}))
+    return df
 
 
 def IWPDCY(config=None):
@@ -351,25 +388,26 @@ def Capacity_stats(raw=False, level=2, config=None, update=False,
         config = get_config()
 
     df = parse_if_not_stored('Capacity_stats', update, config, index_col=0)
-    if raw: return df
+    if raw:
+        return df
 
     countries = config['target_countries']
     df = (df.query('source == @source & year == @year')
           .rename(columns={'technology': 'Fueltype'}).rename(columns=str.title)
           .powerplant.convert_alpha2_to_country()
-#          .query('Country in @countries')
+          # .query('Country in @countries')
           .replace(dict(Fueltype={
-                  'Bioenergy and other renewable fuels': 'Bioenergy',
-                  'Bioenergy and renewable waste': 'Waste',
-                  'Coal derivatives': 'Hard Coal',
-                  'Differently categorized fossil fuels': 'Other',
-                  'Differently categorized renewable energy sources':
-                  'Other',
-                  'Hard coal': 'Hard Coal',
-                  'Mixed fossil fuels': 'Other',
-                  'Natural gas': 'Natural Gas',
-                  'Other or unspecified energy sources': 'Other',
-                  'Tide, wave, and ocean': 'Other'}))
+              'Bioenergy and other renewable fuels': 'Bioenergy',
+              'Bioenergy and renewable waste': 'Waste',
+              'Coal derivatives': 'Hard Coal',
+              'Differently categorized fossil fuels': 'Other',
+              'Differently categorized renewable energy sources':
+              'Other',
+              'Hard coal': 'Hard Coal',
+              'Mixed fossil fuels': 'Other',
+              'Natural gas': 'Natural Gas',
+              'Other or unspecified energy sources': 'Other',
+              'Tide, wave, and ocean': 'Other'}))
           .loc[lambda df: df.Fueltype.isin(config['target_fueltypes'])]
           .pipe(set_column_name, source.title()))
     return df
@@ -388,13 +426,12 @@ def GPD(raw=False, filter_other_dbs=True, update=False, config=None):
         defaults to powerplantmatching.config.get_config()
 
     """
-
-
     config = get_config() if config is None else config
 
-    #if outdated have a look at
-    #http://datasets.wri.org/dataset/globalpowerplantdatabase
+    # if outdated have a look at
+    # http://datasets.wri.org/dataset/globalpowerplantdatabase
     url = config['GPD']['url']
+
     def parse_func():
         import requests
         from zipfile import ZipFile
@@ -404,15 +441,16 @@ def GPD(raw=False, filter_other_dbs=True, update=False, config=None):
             content = parse.content
         else:
             IOError(f'URL {url} seems to be outdated, please doulble the '
-                          'address at http://datasets.wri.org/dataset/'
-                          'globalpowerplantdatabase update the and update the '
-                          'url in your custom config file '
-                          '{package_config["custom_config"]}')
+                    'address at http://datasets.wri.org/dataset/'
+                    'globalpowerplantdatabase update the and update the '
+                    'url in your custom config file '
+                    '{package_config["custom_config"]}')
         return pd.read_csv(ZipFile(BytesIO(content))
                            .open('global_power_plant_database.csv'))
 
     df = parse_if_not_stored('GPD', update, config, parse_func, index_col=0)
-    if raw: return df
+    if raw:
+        return df
 
     other_dbs = []
     if filter_other_dbs:
@@ -428,22 +466,22 @@ def GPD(raw=False, filter_other_dbs=True, update=False, config=None):
                              'Latitude': 'lat',
                              'Longitude': 'lon',
                              'Capacity_Mw': 'Capacity',
-#                             'Source': 'File'
-                             'Commissioning_Year': 'YearCommissioned'})
+                             # 'Source': 'File'
+                             'Commissioning_Year': 'DateIn'})
             .replace(dict(Fueltype={'Coal': 'Hard Coal',
                                     'Biomass': 'Bioenergy',
                                     'Gas': 'Natural Gas',
-                                    'Wave and Tidal': 'Other'}))
+                                    'Wave and Tidal': 'Hydro'}))
             .pipe(clean_powerplantname)
             .pipe(set_column_name, 'GPD')
             .pipe(config_filter, name='GPD', config=config)
-#            .pipe(gather_technology_info, config=config)
-#            .pipe(gather_set_info)
-#            .pipe(correct_manually, 'GPD', config=config)
+            # .pipe(gather_technology_info, config=config)
+            # .pipe(gather_set_info)
+            # .pipe(correct_manually, 'GPD', config=config)
             )
 
 
-#def WIKIPEDIA(raw=False):
+# def WIKIPEDIA(raw=False):
 #    from bs4 import BeautifulSoup
 #
 #    url = 'https://en.wikipedia.org/wiki/List_of_power_stations_in_Germany'
@@ -454,7 +492,7 @@ def GPD(raw=False, filter_other_dbs=True, update=False, config=None):
 #    headers = [header[:-6] for header in all_headers if header[-6:] == '[edit]']
 #    headers = headers[:len(dfs)]
 #    df = pd.concat(dfs, keys=headers, axis=0, sort=True)
-#
+
 
 def ESE(raw=False, update=False, config=None):
     """
@@ -477,7 +515,8 @@ def ESE(raw=False, update=False, config=None):
     """
     config = get_config() if config is None else config
     df = parse_if_not_stored('ESE', update, config, error_bad_lines=False)
-    if raw: return df
+    if raw:
+        return df
 
     target_countries = config['target_countries']
     return (df.rename(columns=str.strip)
@@ -488,9 +527,9 @@ def ESE(raw=False, update=False, config=None):
                              'Technology Broad Category': 'Fueltype'})
             .assign(Set='Store',
                     projectID='ESE' + df.index.astype(str),
-                    YearCommissioned=lambda df: df['Commissioned'].str[-4:]
-                                .apply(pd.to_numeric, errors='coerce'),
-                    Capacity = df['Rated Power'] /1e3)
+                    DateIn=lambda df: (df['Commissioned'].str[-4:]
+                                       .apply(pd.to_numeric, errors='coerce')),
+                    Capacity=df['Rated Power'] / 1e3)
             .query("Status == 'Operational' & Country in @target_countries")
             .pipe(clean_powerplantname)
             .pipe(clean_technology, generalize_hydros=True)
@@ -498,9 +537,10 @@ def ESE(raw=False, update=False, config=None):
                                     u'Pumped Hydro Storage': 'Hydro'}))
             .pipe(set_column_name, 'ESE')
             .pipe(config_filter, name='ESE', config=config)
-#            .pipe(correct_manually, 'ESE', config=config)
+            # .pipe(correct_manually, 'ESE', config=config)
             )
 
+
 def ENTSOE(update=False, raw=False, entsoe_token=None, config=None):
     """
     Importer for the list of installed generators provided by the ENTSO-E
@@ -552,19 +592,17 @@ def ENTSOE(update=False, raw=False, entsoe_token=None, config=None):
         entsoe = pd.DataFrame()
         logger.info(f"Retrieving data from {url}")
         for domain in domains:
-            ret = requests.get(url, params=dict(securityToken=entsoe_token,
-                                           documentType='A71',
-                                           processType='A33',
-                                           In_Domain=domain,
-                                           periodStart='201612312300',
-                                           periodEnd='201712312300'))
+            ret = requests.get(url, params=dict(
+                securityToken=entsoe_token, documentType='A71',
+                processType='A33', In_Domain=domain,
+                periodStart='201612312300', periodEnd='201712312300'))
             etree = ET.fromstring(ret.content)
             ns = namespace(etree)
             df_domain = pd.DataFrame(columns=level1+level2+level3+['Country'])
             for i, level in enumerate([level1, level2, level3]):
                 for arg in level:
-                    df_domain[arg] = [e.text for e
-                      in etree.findall('*/'* (i+1) + ns + arg)]
+                    df_domain[arg] = [
+                        e.text for e in etree.findall('*/' * (i+1) + ns + arg)]
             entsoe = entsoe.append(df_domain, ignore_index=True)
         return entsoe
 
@@ -577,7 +615,8 @@ def ENTSOE(update=False, raw=False, entsoe_token=None, config=None):
                         'falling back to stored version.')
         df = parse_if_not_stored('ENTSOE', update, config)
 
-    if raw: return df
+    if raw:
+        return df
 
     fuelmap = entsoe_api.mappings.PSRTYPE_MAPPINGS
     country_map_entsoe = pd.read_csv(_package_data('entsoe_country_codes.csv'),
@@ -585,32 +624,31 @@ def ENTSOE(update=False, raw=False, entsoe_token=None, config=None):
     countries = config['target_countries']
 
     return (df.rename(columns={'psrType': 'Fueltype',
-                             'quantity': 'Capacity',
-                             'registeredResource.mRID': 'projectID',
-                             'registeredResource.name': 'Name'})
+                               'quantity': 'Capacity',
+                               'registeredResource.mRID': 'projectID',
+                               'registeredResource.name': 'Name'})
             .reindex(columns=config['target_columns'])
             .replace({'Fueltype': fuelmap})
             .drop_duplicates('projectID')
-            .assign(eic_code = lambda df: df.projectID,
+            .assign(EIC=lambda df: df.projectID,
                     Country=lambda df: df.projectID.str[:2]
                                          .map(country_map_entsoe),
                     Name=lambda df: df.Name.str.title(),
                     Fueltype=lambda df: df.Fueltype.replace(
-                            {'Fossil Hard coal': 'Hard Coal',
-                             'Fossil Coal-derived gas': 'Other',
-                             '.*Hydro.*': 'Hydro',
-                             '.*Oil.*': 'Oil',
-                             '.*Peat': 'Bioenergy',
-                             'Fossil Brown coal/Lignite':'Lignite',
-                             'Biomass': 'Bioenergy',
-                             'Fossil Gas': 'Natural Gas',
-                             'Marine': 'Other',
-                             'Wind Offshore': 'Offshore',
-                             'Wind Onshore': 'Onshore'}, regex=True),
+                        {'Fossil Hard coal': 'Hard Coal',
+                         'Fossil Coal-derived gas': 'Other',
+                         '.*Hydro.*': 'Hydro',
+                         '.*Oil.*': 'Oil',
+                         '.*Peat': 'Bioenergy',
+                         'Fossil Brown coal/Lignite': 'Lignite',
+                         'Biomass': 'Bioenergy',
+                         'Fossil Gas': 'Natural Gas',
+                         'Marine': 'Other',
+                         'Wind Offshore': 'Offshore',
+                         'Wind Onshore': 'Onshore'}, regex=True),
                     Capacity=lambda df: pd.to_numeric(df.Capacity))
             .powerplant.convert_alpha2_to_country()
             .pipe(clean_powerplantname)
-#            .query('Country in @countries')
             .pipe(fill_geoposition, use_saved_locations=True, saved_only=True)
             .query('Capacity > 0')
             .pipe(gather_technology_info, config=config)
@@ -618,11 +656,11 @@ def ENTSOE(update=False, raw=False, entsoe_token=None, config=None):
             .pipe(clean_technology)
             .pipe(set_column_name, 'ENTSOE')
             .pipe(config_filter, name='ENTSOE', config=config)
-#            .pipe(correct_manually, 'ENTSOE', config=config)
+            .pipe(correct_manually, 'ENTSOE', config=config)
             )
 
 
-#def OSM():
+# def OSM():
 #    """
 #    Parser and Importer for Open Street Map power plant data.
 #    """
@@ -696,12 +734,12 @@ def WEPP(raw=False, config=None):
                          'Fuel': 'Fueltype',
                          'Fueltype': 'Technology',
                          'Mw': 'Capacity',
-                         'Year': 'YearCommissioned',
-                         #  'Retire':'YearDecommissioned',
+                         'Year': 'DateIn',
+                         'Retire': 'DateOut',
                          'Lat': 'lat',
                          'Lon': 'lon',
                          'Unitid': 'projectID'}, inplace=True)
-    wepp.loc[:, 'Retrofit'] = wepp.YearCommissioned
+    wepp.loc[:, 'DateRetrofit'] = wepp.DateIn
     # Do country transformations and drop those which are not in definded scope
     c = {'ENGLAND & WALES': u'UNITED KINGDOM',
          'GIBRALTAR': u'SPAIN',
@@ -836,50 +874,50 @@ def UBA(header=9, skipfooter=26, prune_wind=True, prune_solar=True,
     """
     config = get_config() if config is None else config
 
-    parse_func = lambda url: pd.read_excel(url, skipfooter=skipfooter,
-                                           na_values='n.b.', header=header)
+    def parse_func(url): return pd.read_excel(url, skipfooter=skipfooter,
+                                              na_values='n.b.', header=header)
     uba = parse_if_not_stored('UBA', update, config, parse_func)
     if raw:
         return uba
     uba = uba.rename(columns={
-            u'Kraftwerksname / Standort': 'Name',
-            u'Elektrische Bruttoleistung (MW)': 'Capacity',
-            u'Inbetriebnahme  (ggf. Ertüchtigung)': 'YearCommissioned',
-            u'Primärenergieträger': 'Fueltype',
-            u'Anlagenart': 'Technology',
-            u'Fernwärme-leistung (MW)': 'CHP',
-            u'Standort-PLZ': 'PLZ'})
+        u'Kraftwerksname / Standort': 'Name',
+        u'Elektrische Bruttoleistung (MW)': 'Capacity',
+        u'Inbetriebnahme  (ggf. Ertüchtigung)': 'DateIn',
+        u'Primärenergieträger': 'Fueltype',
+        u'Anlagenart': 'Technology',
+        u'Fernwärme-leistung (MW)': 'CHP',
+        u'Standort-PLZ': 'PLZ'})
     from .heuristics import PLZ_to_LatLon_map
     uba = (uba.assign(
-            Name=uba.Name.replace({'\s\s+': ' '}, regex=True),
-            lon=uba.PLZ.map(PLZ_to_LatLon_map()['lon']),
-            lat=uba.PLZ.map(PLZ_to_LatLon_map()['lat']),
-            YearCommissioned=uba.YearCommissioned.str.replace(
-                    "\(|\)|\/|\-", " ").str.split(' ').str[0].astype(float),
-            Country='Germany',
-            File='kraftwerke-de-ab-100-mw.xls',
-            projectID=['UBA{:03d}'.format(i + header + 2) for i in uba.index],
-            Technology=uba.Technology.replace({
-                    u'DKW': 'Steam Turbine',
-                    u'DWR': 'Pressurized Water Reactor',
-                    u'G/AK': 'Steam Turbine',
-                    u'GT': 'OCGT',
-                    u'GuD': 'CCGT',
-                    u'GuD / HKW': 'CCGT',
-                    u'HKW': 'Steam Turbine',
-                    u'HKW (DT)': 'Steam Turbine',
-                    u'HKW / GuD': 'CCGT',
-                    u'HKW / SSA': 'Steam Turbine',
-                    u'IKW': 'OCGT',
-                    u'IKW / GuD': 'CCGT',
-                    u'IKW / HKW': 'Steam Turbine',
-                    u'IKW / HKW / GuD': 'CCGT',
-                    u'IKW / SSA': 'OCGT',
-                    u'IKW /GuD': 'CCGT',
-                    u'LWK': 'Run-Of-River',
-                    u'PSW': 'Pumped Storage',
-                    u'SWK': 'Reservoir Storage',
-                    u'SWR': 'Boiled Water Reactor'})))
+        Name=uba.Name.replace({'\s\s+': ' '}, regex=True),
+        lon=uba.PLZ.map(PLZ_to_LatLon_map()['lon']),
+        lat=uba.PLZ.map(PLZ_to_LatLon_map()['lat']),
+        DateIn=uba.DateIn.str.replace(
+            "\(|\)|\/|\-", " ").str.split(' ').str[0].astype(float),
+        Country='Germany',
+        File='kraftwerke-de-ab-100-mw.xls',
+        projectID=['UBA{:03d}'.format(i + header + 2) for i in uba.index],
+        Technology=uba.Technology.replace({
+            u'DKW': 'Steam Turbine',
+            u'DWR': 'Pressurized Water Reactor',
+            u'G/AK': 'Steam Turbine',
+            u'GT': 'OCGT',
+            u'GuD': 'CCGT',
+            u'GuD / HKW': 'CCGT',
+            u'HKW': 'Steam Turbine',
+            u'HKW (DT)': 'Steam Turbine',
+            u'HKW / GuD': 'CCGT',
+            u'HKW / SSA': 'Steam Turbine',
+            u'IKW': 'OCGT',
+            u'IKW / GuD': 'CCGT',
+            u'IKW / HKW': 'Steam Turbine',
+            u'IKW / HKW / GuD': 'CCGT',
+            u'IKW / SSA': 'OCGT',
+            u'IKW /GuD': 'CCGT',
+            u'LWK': 'Run-Of-River',
+            u'PSW': 'Pumped Storage',
+            u'SWK': 'Reservoir Storage',
+            u'SWR': 'Boiled Water Reactor'})))
     uba.loc[uba.CHP.notnull(), 'Set'] = 'CHP'
     uba = uba.pipe(gather_set_info)
     uba.loc[uba.Fueltype == 'Wind (O)', 'Technology'] = 'Offshore'
@@ -907,8 +945,8 @@ def UBA(header=9, skipfooter=26, prune_wind=True, prune_solar=True,
     return (uba
             .pipe(set_column_name, 'UBA')
             .pipe(scale_to_net_capacities, not config['UBA']['net_capacity'])
-#            .pipe(config_filter, name='UBA', config=config)
-#            .pipe(correct_manually, 'UBA', config=config)
+            # .pipe(config_filter, name='UBA', config=config)
+            # .pipe(correct_manually, 'UBA', config=config)
             )
 
 
@@ -937,28 +975,30 @@ def BNETZA(header=9, sheet_name='Gesamtkraftwerksliste BNetzA',
     """
     config = get_config() if config is None else config
 
-    parse_func = lambda url: pd.read_excel(url, header=header,
-                                           sheet_name=sheet_name,
-                                           parse_dates=False)
+    url = config['BNETZA']['url']
+
+    def parse_func():
+        return pd.read_excel(url, header=header, sheet_name=sheet_name,
+                             parse_dates=False)
     bnetza = parse_if_not_stored('BNETZA', update, config, parse_func)
 
     if raw:
         return bnetza
     bnetza = bnetza.rename(columns={
-            u'Kraftwerksnummer Bundesnetzagentur': 'projectID',
-            u'Kraftwerksname': 'Name',
-            u'Netto-Nennleistung (elektrische Wirkleistung) in MW': 'Capacity',
-            u'Wärmeauskopplung (KWK)\n(ja/nein)': 'Set',
-            u'Ort\n(Standort Kraftwerk)': 'Ort',
-            (u'Auswertung\nEnergieträger (Zuordnung zu einem '
-             u'Hauptenergieträger bei Mehreren Energieträgern)'): 'Fueltype',
-           'Kraftwerksstatus \n(in Betrieb/\nvorläufig '
-           'stillgelegt/\nsaisonale Konservierung\nNetzreserve/ '
-           'Sicherheitsbereitschaft/\nSonderfall)': 'Status',
-            ('Aufnahme der kommerziellen Stromerzeugung der derzeit '
-             'in Betrieb befindlichen Erzeugungseinheit\n(Datum/Jahr)'):
-            'YearCommissioned',
-            u'PLZ\n(Standort Kraftwerk)': 'PLZ'})
+        'Kraftwerksnummer Bundesnetzagentur': 'projectID',
+        'Kraftwerksname': 'Name',
+        'Netto-Nennleistung (elektrische Wirkleistung) in MW': 'Capacity',
+        'Wärmeauskopplung (KWK)\n(ja/nein)': 'Set',
+        'Ort\n(Standort Kraftwerk)': 'Ort',
+        ('Auswertung\nEnergieträger (Zuordnung zu einem '
+         'Hauptenergieträger bei Mehreren Energieträgern)'): 'Fueltype',
+        'Kraftwerksstatus \n(in Betrieb/\nvorläufig '
+        'stillgelegt/\nsaisonale Konservierung\nNetzreserve/ '
+        'Sicherheitsbereitschaft/\nSonderfall)': 'Status',
+        ('Aufnahme der kommerziellen Stromerzeugung der derzeit '
+         'in Betrieb befindlichen Erzeugungseinheit\n(Datum/Jahr)'):
+        'DateIn',
+        'PLZ\n(Standort Kraftwerk)': 'PLZ'})
     # If BNetzA-Name is empty replace by company, if this is empty by city.
 
     from .heuristics import PLZ_to_LatLon_map
@@ -969,19 +1009,18 @@ def BNETZA(header=9, sheet_name='Gesamtkraftwerksliste BNetzA',
               lon=bnetza.PLZ.map(PLZ_to_LatLon_map()['lon']),
               lat=bnetza.PLZ.map(PLZ_to_LatLon_map()['lat']),
               Name=bnetza.Name.where(bnetza.Name.str.len().fillna(0) > 4,
-                                     bnetza.Unternehmen + ' ' +
-                                     bnetza.Name.fillna(''))
-                              .fillna(bnetza.Ort).str.strip(),
-              YearCommissioned=
-                  bnetza.YearCommissioned.str[:4]
-                        .apply(pd.to_numeric, errors='coerce'),
+                                     bnetza.Unternehmen + ' '
+                                     + bnetza.Name.fillna(''))
+              .fillna(bnetza.Ort).str.strip(),
+              DateIn=bnetza.DateIn.str[:4]
+              .apply(pd.to_numeric, errors='coerce'),
               Blockname=bnetza.Blockname.replace(
-                      {'.*(GT|gasturbine).*': 'OCGT',
-                       '.*(DT|HKW|(?i)dampfturbine|(?i)heizkraftwerk).*':
-                           'Steam Turbine',
-                       '.*GuD.*': 'CCGT'}, regex=True))
-              [lambda df: df.projectID.notna() &
-               df.Status.str.contains(pattern, regex=True, case=False)]
+                  {'.*(GT|gasturbine).*': 'OCGT',
+                   '.*(DT|HKW|(?i)dampfturbine|(?i)heizkraftwerk).*':
+                       'Steam Turbine',
+                   '.*GuD.*': 'CCGT'}, regex=True))
+              [lambda df: df.projectID.notna()
+               & df.Status.str.contains(pattern, regex=True, case=False)]
               .pipe(gather_technology_info,
                     search_col=['Name', 'Fueltype', 'Blockname'],
                     config=config))
@@ -991,9 +1030,9 @@ def BNETZA(header=9, sheet_name='Gesamtkraftwerksliste BNetzA',
                              and (str.title(ds['Ort']) not in ds['Name']),
                              axis=1))
     bnetza.loc[bnetza.Ort.notnull() & add_location_b, 'Name'] = (
-                bnetza.loc[bnetza.Ort.notnull() & add_location_b, 'Ort']
-                + ' '
-                + bnetza.loc[bnetza.Ort.notnull() & add_location_b, 'Name'])
+        bnetza.loc[bnetza.Ort.notnull() & add_location_b, 'Ort']
+        + ' '
+        + bnetza.loc[bnetza.Ort.notnull() & add_location_b, 'Name'])
 
     techmap = {'solare': 'PV',
                'Laufwasser': 'Run-Of-River',
@@ -1015,8 +1054,8 @@ def BNETZA(header=9, sheet_name='Gesamtkraftwerksliste BNetzA',
                              '.*(?i)(e|r|n)gas': 'Other',
                              'Geoth.*': 'Geothermal',
                              'Abfall': 'Waste',
-                             '.*wasser.*':'Hydro',
-                             '.*solar.*':'PV'},
+                             '.*wasser.*': 'Hydro',
+                             '.*solar.*': 'PV'},
                             regex=True, inplace=True)
     if prune_wind:
         bnetza = bnetza[lambda x: x.Fueltype != 'Wind']
@@ -1027,10 +1066,10 @@ def BNETZA(header=9, sheet_name='Gesamtkraftwerksliste BNetzA',
                                              'Luxemburg'])]
     return (bnetza.assign(Country='Germany',
                           Set=bnetza.Set.fillna('Nein').str.title()
-                              .replace({'Ja': 'CHP', 'Nein': 'PP'}))
+                          .replace({'Ja': 'CHP', 'Nein': 'PP'}))
             .pipe(set_column_name, 'BNETZA')
-#            .pipe(config_filter, name='BNETZA', config=config)
-#            .pipe(correct_manually, 'BNETZA', config=config)
+            # .pipe(config_filter, name='BNETZA', config=config)
+            # .pipe(correct_manually, 'BNETZA', config=config)
             )
 
 
@@ -1053,7 +1092,8 @@ def OPSD_VRE(config=None, raw=False):
     config = get_config() if config is None else config
 
     df = parse_if_not_stored('OPSD_VRE', index_col=0, low_memory=False)
-    if raw: return df
+    if raw:
+        return df
 
     return df.rename(columns={'energy_source_level_2': 'Fueltype',
                               'technology': 'Technology',
@@ -1061,34 +1101,35 @@ def OPSD_VRE(config=None, raw=False):
                               'country': 'Country',
                               'electrical_capacity': 'Capacity',
                               'municipality': 'Name'})\
-            .assign(YearCommissioned=lambda df:
-                    df.commissioning_date.str[:4].astype(float))\
-            .powerplant.convert_alpha2_to_country()\
-            .pipe(set_column_name, 'OPSD_VRE')\
-            .pipe(config_filter, config=config)\
-            .drop('Name', axis=1)
+        .assign(DateIn=lambda df:
+                df.commissioning_date.str[:4].astype(float),
+                Set='PP')\
+        .powerplant.convert_alpha2_to_country()\
+        .pipe(set_column_name, 'OPSD_VRE')\
+        .pipe(config_filter, config=config)\
+        .drop('Name', axis=1)
 
 
 def OPSD_VRE_country(country, config=None, raw=False):
     """
-    Get country specifig data from OPSD for renewables. For using this,
-    the config has to be adjusted.
+    Get country specifig data from OPSD for renewables, if available.
+    Available for DE, FR, PL, CH, DK, CZ and SE (last update: 09/2020).
     """
     config = get_config() if config is None else config
 
-    df = parse_if_not_stored(f'OPSD_VRE_{country}')
-    if raw: return df
+    df = parse_if_not_stored(f'OPSD_VRE_{country}', engine='python')
+    if raw:
+        return df
 
-    return (df.assign(Country = 'DE')
-            .rename(columns={'energy_source_level_2': 'Fueltype',
-                             'technology': 'Technology',
-                             'data_source': 'file',
-                             'country': 'Country',
-                             'electrical_capacity': 'Capacity',
-                             'municipality': 'Name'})\
-                .powerplant.convert_alpha2_to_country()\
-                .pipe(config_filter, config=config)\
-                .drop('Name', axis=1))
+    return (df.assign(Country=country, Set='PP')
+              .rename(columns={'energy_source_level_2': 'Fueltype',
+                               'technology': 'Technology',
+                               'data_source': 'file',
+                               'electrical_capacity': 'Capacity',
+                               'municipality': 'Name'})
+              .powerplant.convert_alpha2_to_country()
+              .pipe(config_filter, config=config)
+              .drop('Name', axis=1))
 
 
 def IRENA_stats(config=None):
