@@ -65,6 +65,10 @@ def extend_by_non_matched(df, extend_by, label=None, query=None,
     else:
         included_ids = (df.projectID.dropna().map(lambda d: d.get(label))
                           .dropna().sum())
+    if included_ids == 0:
+        logger.warning(f'{label} not existent in the matched date, extending'
+                       ' by all data entries.')
+        included_ids = []
 
     if query is not None:
         extend_by.query(query, inplace=True)
@@ -74,16 +78,17 @@ def extend_by_non_matched(df, extend_by, label=None, query=None,
         extend_by = aggregate_units(extend_by, dataset_name=label,
                                     config=config, **aggkwargs)
         extend_by = extend_by.assign(
-                projectID=extend_by.projectID.map(lambda x: {label: x}))
+            projectID=extend_by.projectID.map(lambda x: {label: x}))
     else:
         extend_by = extend_by.assign(
-                projectID=extend_by.projectID.map(lambda x: {label: [x]}))
+            projectID=extend_by.projectID.map(lambda x: {label: [x]}))
 
     if df.columns.nlevels > 1:
         return df.append(
-                pd.concat([extend_by], keys=[label], axis=1) #, ignore_index=True ??
-                .swaplevel(axis=1)
-                .reindex(columns=df.columns), ignore_index=True)
+            # , ignore_index=True ??
+            pd.concat([extend_by], keys=[label], axis=1)
+            .swaplevel(axis=1)
+            .reindex(columns=df.columns), ignore_index=True)
     else:
         return df.append(extend_by.reindex(columns=df.columns),
                          ignore_index=True)
@@ -159,10 +164,10 @@ def extend_by_VRE(df, config=None, base_year=2017, prune_beyond=True):
     config = get_config() if config is None else config
 
     vre = OPSD_VRE(config=config).query('Fueltype != "Hydro"')\
-            .reindex(columns=config['target_columns'])
+        .reindex(columns=config['target_columns'])
     return df.append(vre, sort=False)
 
-#I am sorry to drop this, but this is a bit too specific for maintaining
+# I am sorry to drop this, but this is a bit too specific for maintaining
 #
 #    df = df.copy()
 #    # Drop Solar (except CSP), Wind and Bioenergy which are to be replaced
@@ -197,8 +202,8 @@ def extend_by_VRE(df, config=None, base_year=2017, prune_beyond=True):
 #    cc = pd.concat([df, vre_DK, vre_CH_DE, vre], ignore_index=True, sort=False)
 #    cc = cc.loc[:, cols]
 #    if prune_beyond:
-#        cc = cc[(cc.YearCommissioned <= base_year) |
-#                (cc.YearCommissioned.isnull())]
+#        cc = cc[(cc.DateIn <= base_year) |
+#                (cc.DateIn.isnull())]
 #    cc.reset_index(drop=True, inplace=True)
 #    return cc
 
@@ -210,41 +215,41 @@ def fill_missing_commyears(df):
     df = get_obj_if_Acc(df)
     df = df.copy()
     # 1st try: Fill with both country- and fueltypespecific averages
-    df.YearCommissioned.fillna(df.groupby(['Country', 'Fueltype'])
-                                 .YearCommissioned
+    df.DateIn.fillna(df.groupby(['Country', 'Fueltype'])
+                                 .DateIn
                                  .transform('mean'), inplace=True)
     # 2nd try: Fill remaining with only fueltype-specific average
-    df.YearCommissioned.fillna(df.groupby(['Fueltype']).YearCommissioned
+    df.DateIn.fillna(df.groupby(['Fueltype']).DateIn
                                  .transform('mean'), inplace=True)
     # 3rd try: Fill remaining with only country-specific average
-    df.YearCommissioned.fillna(df.groupby(['Country']).YearCommissioned
+    df.DateIn.fillna(df.groupby(['Country']).DateIn
                                  .transform('mean'), inplace=True)
-    if df.YearCommissioned.isnull().any():
-        count = len(df[df.YearCommissioned.isnull()])
+    if df.DateIn.isnull().any():
+        count = len(df[df.DateIn.isnull()])
         raise(ValueError('''There are still *{0}* empty values for
-                            'YearCommissioned' in the DataFrame. These should
+                            'DateIn' in the DataFrame. These should
                             be either be filled manually or dropped to
                             continue.'''.format(count)))
-    df.loc[:, 'YearCommissioned'] = df.YearCommissioned.astype(int)
-    df.Retrofit.fillna(df.YearCommissioned.astype(int), inplace=True)
+    df.loc[:, 'DateIn'] = df.DateIn.astype(int)
+    df.DateRetrofit.fillna(df.DateIn.astype(int), inplace=True)
     return df
 
 
 def fill_missing_decommyears(df, config=None):
     """
-    Function which sets/fills a column 'YearDecommissioning' with roughly
+    Function which sets/fills a column 'DateOut' with roughly
     estimated values for decommissioning years, based on the estimated lifetimes
     per `Fueltype` given in the config and corresponding commissioning years.
     Note that the latter is filled up using `fill_missing_commyears`.
     """
     df = get_obj_if_Acc(df)
-    if config is None: config = get_config()
-    if 'YearDecommissioning' not in df:
-        df = df.reindex(columns = list(df.columns) + ['YearDecommissioning'])
+    if config is None:
+        config = get_config()
+    if 'DateOut' not in df:
+        df = df.reindex(columns=list(df.columns) + ['DateOut'])
     lifetime = df.Fueltype.map(config['fuel_to_lifetime'])
     df = fill_missing_commyears(df)
-    df['YearDecommissioning'] = (df.YearDecommissioning
-                                 .fillna(df[['YearCommissioned', 'Retrofit']].max(1)
+    df['DateOut'] = (df.DateOut.fillna(df[['DateIn', 'DateRetrofit']].max(1)
                                          + lifetime).astype(int))
     return df
 
@@ -273,7 +278,7 @@ def aggregate_VRE_by_commyear(df, target_fueltypes=None, agg_geo_by=None):
         f = {'Capacity': ['sum'], 'lat': ['mean'], 'lon': ['mean']}
     elif agg_geo_by == 'wm':
         # TODO: This does not work yet, when NaNs are in lat/lon columns.
-        wm = lambda x: np.average(x, weights=df.loc[x.index, 'Capacity'])
+        def wm(x): return np.average(x, weights=df.loc[x.index, 'Capacity'])
         f = {'Capacity': ['sum'],
              'lat': {'weighted mean': wm},
              'lon': {'weighted mean': wm}}
@@ -286,11 +291,11 @@ def aggregate_VRE_by_commyear(df, target_fueltypes=None, agg_geo_by=None):
     df = df[df.Fueltype.isin(target_fueltypes)]
     df = fill_missing_commyears(df)
     df.Technology.fillna('-', inplace=True)
-    df = (df.groupby(['Country', 'YearCommissioned', 'Fueltype', 'Technology'])
+    df = (df.groupby(['Country', 'DateIn', 'Fueltype', 'Technology'])
             .agg(f).reset_index().replace({'-': np.NaN}))
     df.columns = df.columns.droplevel(level=1)
     return df.assign(Set='PP',
-                     Retrofit=df.YearCommissioned)
+                     DateRetrofit=df.DateIn)
 
 
 def derive_vintage_cohorts_from_statistics(df, base_year=2015, config=None):
@@ -359,13 +364,13 @@ def derive_vintage_cohorts_from_statistics(df, base_year=2015, config=None):
     dfe = pd.DataFrame(columns=df.columns)
     for c, df_country in df.groupby(['Country']):
         for tech, dfs in df_country.groupby(['Technology']):
-            dfs.set_index('YearCommissioned', drop=False, inplace=True)
+            dfs.set_index('DateIn', drop=False, inplace=True)
             y_start = int(dfs.index[0])
             y_end = int(dfs.index[-1])
             life = config['fuel_to_lifetime'][dfs.Fueltype.iloc[0]]
             mat = (pd.DataFrame(columns=range(y_start-life+1, y_end+life),
                                 index=range(y_start-life+1, y_end))
-                     .astype(np.float))
+                   .astype(np.float))
             if dfs.Fueltype.iloc[0] in ['Solar', 'Wind', 'Bioenergy',
                                         'Geothermal']:
                 mat = setInitial_Triangle(mat, dfs, life)
@@ -382,8 +387,8 @@ def derive_vintage_cohorts_from_statistics(df, base_year=2015, config=None):
             add.Set = dfs.Set.iloc[0]
             dfe = pd.concat([dfe, add[add.Capacity > 0.0]], ignore_index=True)
     dfe.Year = dfe.Year.apply(pd.to_numeric)
-    dfe.rename(columns={'Year': 'YearCommissioned'}, inplace=True)
-    dfe = dfe.assign(Retrofit=dfe.YearCommissioned)
+    dfe.rename(columns={'Year': 'DateIn'}, inplace=True)
+    dfe = dfe.assign(DateRetrofit=dfe.DateIn)
     return dfe[~np.isclose(dfe.Capacity, 0)]
 
 
@@ -453,15 +458,15 @@ def gross_to_net_factors(reference='opsd', aggfunc='median',
     else:
         df.energy_source_level_2.fillna(value=df.fuel, inplace=True)
         df.replace(dict(energy_source_level_2={
-                'Biomass and biogas': 'Bioenergy',
-                'Fossil fuels': 'Other',
-                'Mixed fossil fuels': 'Other',
-                'Natural gas': 'Natural Gas',
-                'Non-renewable waste': 'Waste',
-                'Other bioenergy and renewable waste': 'Bioenergy',
-                'Other or unspecified energy sources': 'Other',
-                'Other fossil fuels': 'Other',
-                'Other fuels': 'Other'}), inplace=True)
+            'Biomass and biogas': 'Bioenergy',
+            'Fossil fuels': 'Other',
+            'Mixed fossil fuels': 'Other',
+            'Natural gas': 'Natural Gas',
+            'Non-renewable waste': 'Waste',
+            'Other bioenergy and renewable waste': 'Bioenergy',
+            'Other or unspecified energy sources': 'Other',
+            'Other fossil fuels': 'Other',
+            'Other fuels': 'Other'}), inplace=True)
         df.rename(columns={'technology': 'Technology'}, inplace=True)
         df = (clean_technology(df)
               .assign(energy_source_level_2=lambda df:
