@@ -39,14 +39,13 @@ cartopy_present = True
 try:
     import cartopy.crs as ccrs
     import cartopy
-except:
+except (ModuleNotFoundError, ImportError):
     cartopy_present = False
 
 if not cartopy_present:
     logger.warn('Cartopy not existent.')
 
 
-
 def fueltype_stats(df):
     df = get_obj_if_Acc(df)
     stats = lookup(set_uncommon_fueltypes_to_other(df), by='Fueltype')
@@ -76,15 +75,15 @@ def powerplant_map(df, scale=1e2, european_bounds=True, fillcontinents=False,
         legendcols = (pd.Series(get_config()['fuel_to_color'])
                         .reindex(shown_fueltypes))
         handles = sum(legendcols.apply(lambda x:
-                      make_legend_circles_for([10.], scale=scale*legendscale,
-                                              facecolor=x)).tolist(), [])
+                                       make_legend_circles_for([10.], scale=scale*legendscale,
+                                                               facecolor=x)).tolist(), [])
         ax.legend(handles, legendcols.index,
-                   handler_map=make_handler_map_to_scale_circles_as_in(ax),
-                   markerscale=1,
-                   ncol=kwargs.get('ncol', 2),
-                   loc=kwargs.get('loc', "upper left"),
-                   frameon=True, fancybox=True, edgecolor='w',
-                   facecolor='w', framealpha=1)
+                  handler_map=make_handler_map_to_scale_circles_as_in(ax),
+                  markerscale=1,
+                  ncol=kwargs.get('ncol', 2),
+                  loc=kwargs.get('loc', "upper left"),
+                  frameon=True, fancybox=True, edgecolor='w',
+                  facecolor='w', framealpha=1)
 
         ax.set_xlabel('')
         ax.set_ylabel('')
@@ -100,6 +99,7 @@ def powerplant_map(df, scale=1e2, european_bounds=True, fillcontinents=False,
             ax.outline_patch.set_visible(False)
         return fig, ax
 
+
 # This approach is an alternative to bar_comparison_countries_fueltypes()
 def fueltype_and_country_totals_bar(dfs, keys=None, figsize=(18, 8)):
     dfs = get_obj_if_Acc(dfs)
@@ -176,8 +176,8 @@ def factor_comparison(dfs, keys=None, figsize=(12, 9)):
         dfs = [set_uncommon_fueltypes_to_other(df) for df in dfs]
         compare = lookup(dfs, keys=keys, exclude=['Solar', 'Wind']).fillna(0.)
         compare = compare.append(
-                pd.concat([compare.groupby(level='Country').sum()],
-                          keys=['Total']).swaplevel()).sort_index()/1000
+            pd.concat([compare.groupby(level='Country').sum()],
+                      keys=['Total']).swaplevel()).sort_index()/1000
         n_countries, n_fueltypes = compare.index.levshape
         c = [get_config()['fuel_to_color'][i] for i in compare.index.levels[1]]
         rcParams["axes.prop_cycle"] = cycler(color=c)
@@ -186,8 +186,8 @@ def factor_comparison(dfs, keys=None, figsize=(12, 9)):
         compare[compare.sum(1) < 0.5] = np.nan
 
         fig, ax = plt.subplots(1, 1, figsize=figsize)
-        compare = (compare.unstack('Country').swaplevel(axis=1).sort_index(axis=1)
-                   .reindex(columns=keys, level=1))
+        compare = (compare.unstack('Country').swaplevel(axis=1)
+                   .sort_index(axis=1).reindex(columns=keys, level=1))
         compare.T.plot(ax=ax, markevery=(0, 2),
                        style='o', markersize=5)
         compare.T.plot(ax=ax, markevery=(1, 2),
@@ -199,8 +199,8 @@ def factor_comparison(dfs, keys=None, figsize=(12, 9)):
             ax.plot(np.array([0, 1]) + (2*i), compare[j].T)
 
         indexhandles = [Line2D([0.4, 0.6], [0.4, 0.6], marker=m, linewidth=0.,
-                        markersize=msize, color='w', markeredgecolor='k',
-                        markeredgewidth=0.5)
+                               markersize=msize, color='w', markeredgecolor='k',
+                               markeredgewidth=0.5)
                         for m, msize in [['o', 5.], ['s', 4.5]]]
         ax.add_artist(ax.legend(handles=indexhandles, labels=keys))
         ax.legend(handles=lgd[0][:len(c)], labels=lgd[1][:len(c)],
@@ -267,9 +267,8 @@ def make_handler_map_to_scale_circles_as_in(ax,
     fig = ax.get_figure()
 
     def axes2pt():
-        return np.diff(
-                ax.transData.transform([(0, 0), (1, 1)]),
-                axis=0)[0] * (72./fig.dpi)
+        return np.diff(ax.transData.transform([(0, 0), (1, 1)]),
+                       axis=0)[0] * (72./fig.dpi)
     ellipses = []
     if not dont_resize_actively:
         def update_width_height(event):
@@ -303,8 +302,8 @@ def draw_basemap(resolution=True, ax=None, country_linewidth=0.3,
         if ax is None:
             ax = plt.gca(projection=ccrs.PlateCarree())
         resolution = '50m' if isinstance(resolution, bool) else resolution
-        assert resolution in ['10m', '50m', '110m'], ('Resolution has to be '
-                             "one of '10m', '50m', '110m'")
+        assert resolution in ['10m', '50m', '110m'],\
+            "Resolution has to be one of '10m', '50m', '110m'."
         ax.set_extent(ax.get_xlim() + ax.get_ylim(),
                       crs=ccrs.PlateCarree())
         ax.coastlines(linewidth=0.4, zorder=-1, resolution=resolution)
@@ -352,8 +351,8 @@ def gather_nrows_ncols(x, orientation='landscape'):
         # Solution 2:
         n, m = calc(k-1, k+1)
         sol2 = {'n': n, 'm': m, 'dif': (m*n) - x}
-        if (((sol1['dif'] <= sol2['dif']) & (sol1['n'] >= 2)) |
-                (x in [7, 13, 14])):
+        if (((sol1['dif'] <= sol2['dif']) & (sol1['n'] >= 2))
+                | (x in [7, 13, 14])):
             n, m = [sol1['n'], sol1['m']]
         else:
             n, m = [sol2['n'], sol2['m']]
@@ -366,18 +365,17 @@ def gather_nrows_ncols(x, orientation='landscape'):
             raise ValueError('Wrong `orientation` given!')
 
 
-
-#%% extra data needed, plots for publication
+# %% extra data needed, plots for publication
 #
 #
-#def Show_all_plots():
+# def Show_all_plots():
 #    comparison_single_matched_bar()
 #    comparison_1dim(by='Country')
 #    comparison_1dim(by='Fueltype')
 #    return
 #
 #
-#def comparison_countries_fueltypes_bar(
+# def comparison_countries_fueltypes_bar(
 #        dfs=None, include_WEPP=True, include_VRE=False, exclude=None,
 #        show_indicators=True, year=2015, **kwargs):
 #    """
@@ -525,7 +523,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    return fig, ax
 #
 #
-#def matched_fueltype_totals_bar(figsize=(9, 4), axes_style='whitegrid'):
+# def matched_fueltype_totals_bar(figsize=(9, 4), axes_style='whitegrid'):
 #    from . import data
 #    from .collection import Carma_ENTSOE_GEO_OPSD_WRI_matched_reduced
 #    matched = set_uncommon_fueltypes_to_other(
@@ -570,7 +568,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #
 #
 #
-#def bar_decomissioning_curves(df=None, ylabel=None, title=None,
+# def bar_decomissioning_curves(df=None, ylabel=None, title=None,
 #                              legend_in_subplots=False):
 #    """
 #    Plots per country a decommissioning curve as a bar chart with
@@ -591,7 +589,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    # Insert periodwise capacities
 #    df.loc[:, 2015] = df.loc[:, 'Capacity']
 #    for yr in range(2020, 2055, 5):
-#        df.loc[yr <= (df.loc[:, 'YearCommissioned']
+#        df.loc[yr <= (df.loc[:, 'DateIn']
 #                      + df.loc[:, 'Life']), yr] = df.loc[:, 'Capacity']
 #        df.loc[:, yr].fillna(0., inplace=True)
 #
@@ -649,7 +647,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    return fig, ax
 #
 #
-#def area_yearcommissioned(dfs, keys, figsize=(7, 5),
+# def area_yearcommissioned(dfs, keys, figsize=(7, 5),
 #                          ylabel='Capacity [$GW$]'):
 #    """
 #    Plots an area chart by commissioning year.
@@ -663,7 +661,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #        figsize : tuple
 #        ylabel : str
 #    """
-#    dfp = [df.pivot_table(values='Capacity', index='YearCommissioned',
+#    dfp = [df.pivot_table(values='Capacity', index='DateIn',
 #                          columns='Fueltype', aggfunc='sum')/1000
 #           for df in dfs]
 #    labels_mpatches = collections.OrderedDict()
@@ -702,11 +700,11 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    return fig, ax
 #
 #
-## %% Plot utilities
+# %% Plot utilities
 #
 #
 #
-#def comparison_single_matched_bar(df=None, include_WEPP=True, cleaned=True,
+# def comparison_single_matched_bar(df=None, include_WEPP=True, cleaned=True,
 #                                  use_saved_aggregation=True, figsize=(9, 5),
 #                                  exclude=['Geothermal', 'Solar', 'Wind'],
 #                                  axes_style='whitegrid'):
@@ -786,7 +784,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    return fig, ax
 #
 #
-#def comparison_1dim(dfs=None, keys=None, by='Country', include_WEPP=True,
+# def comparison_1dim(dfs=None, keys=None, by='Country', include_WEPP=True,
 #                    include_VRE=False, year=2016, how='hbar',
 #                    axes_style='whitegrid',
 #                    exclude=['Geothermal', 'Solar', 'Wind', 'Battery'],
@@ -856,7 +854,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #        return g.fig, g.axes
 #
 #
-#def gather_comparison_data(include_WEPP=True, include_VRE=False, **kwargs):
+# def gather_comparison_data(include_WEPP=True, include_VRE=False, **kwargs):
 #    from .data import WEPP, Capacity_stats
 #    from .collection import (
 #            matched_data,
@@ -868,7 +866,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    s = "Fueltype!='Solar' and Fueltype!='Wind' and Fueltype!='Geothermal'"
 #    queryexpr = kwargs.get('queryexpr', s)
 #    # 1+2: WEPP itself + Reduced w/ WEPP
-#    s = '(YearCommissioned<={:04d}) or (YearCommissioned!=YearCommissioned)'
+#    s = '(DateIn<={:04d}) or (DateIn!=DateIn)'
 #    if include_WEPP:
 #        wepp = WEPP()
 #        wepp.query(s.format(yr), inplace=True)
@@ -902,7 +900,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    return red_w_wepp, red_wo_wepp, wepp, statistics
 #
 #
-#def matchcount_stats(df):
+# def matchcount_stats(df):
 #    """
 #    Plots the number of matches against the number of involved databases,
 #    across all databases.
@@ -913,7 +911,7 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #    return
 #
 #
-#def gather_nrows_ncols(x, orientation='landscape'):
+# def gather_nrows_ncols(x, orientation='landscape'):
 #    """
 #    Derives [nrows, ncols] based on x plots, so that a subplot looks nicely.
 #
@@ -965,6 +963,6 @@ def gather_nrows_ncols(x, orientation='landscape'):
 #
 #
 #
-#orderdedfuels = ['Hydro',  # 'Solar', 'Wind',
+# orderdedfuels = ['Hydro',  # 'Solar', 'Wind',
 #                 'Nuclear', 'Hard Coal', 'Lignite',
 #                 'Oil', 'Natural Gas', 'Other']
