@@ -70,7 +70,7 @@ def lookup(df, keys=None, by='Country, Fueltype', exclude=None, unit='MW'):
         if keys is None:
             keys = [get_name(d) for d in df]
         dfs = pd.concat([lookup_single(a) for a in df], axis=1,
-                         keys=keys, sort=False)
+                        keys=keys, sort=False)
         dfs = dfs.fillna(0.)
         return (dfs/scaling).round(3)
     else:
@@ -140,31 +140,17 @@ def correct_manually(df, name, config=None):
     name : str
         Name of the data source, should be in columns of manual_corrections.csv
     """
-    from .data import data_config
     if config is None:
         config = get_config()
 
-    corrections = pd.read_csv(_package_data('manual_corrections.csv'),
-                               encoding='utf-8',
-                               parse_dates=['last_update'])
+    corrections_fn = _package_data('manual_corrections.csv')
+    corrections = pd.read_csv(corrections_fn)
+
+    corrections = (corrections.query('Source == @name')
+                   .drop(columns='Source').set_index('projectID'))
+    if corrections.empty:
+        return df
 
-    if name in corrections:
-        corrections = corrections[corrections[name].notnull()].set_index(name)
-    else:
-        return df.reindex(columns=config['target_columns'])
-    if len(corrections) == 0:
-        return df.reindex(columns=config['target_columns'])
-    source_file = data_config[name]['source_file']
-    # assume OPSD files are updated on the same time
-    if isinstance(source_file, list):
-        source_file = source_file[0]
-    outdated = (pd.Timestamp(time.ctime(os.path.getmtime(source_file)))
-                > corrections.last_update).any()
-    if outdated:
-        logger.warning('Manual corrections in {0} for file {1} older than last'
-                       ' update of the source file, please update your manual '
-                       'corrections.'.format(os.path.abspath(
-                               _data_in('manual_corrections.csv')), name))
     df = df.set_index('projectID').copy()
     df.update(corrections)
     return df.reset_index().reindex(columns=config['target_columns'])
@@ -207,7 +193,7 @@ def read_csv_if_string(df):
     """
     from . import data
     if isinstance(data, six.string_types):
-        df =getattr(data, df)()
+        df = getattr(data, df)()
     return df
 
 
@@ -222,7 +208,7 @@ def to_categorical_columns(df):
             'Set': get_config()['target_sets']}
     return df.assign(**{c: df[c].astype('category') for c in cols})\
              .assign(**{c: lambda df: df[c].cat.set_categories(v)
-                     for c,v in cats.items()})
+                        for c, v in cats.items()})
 
 
 def set_column_name(df, name):
@@ -271,7 +257,7 @@ def projectID_to_dict(df):
     """
     if df.columns.nlevels > 1:
         return df.assign(projectID=(df.projectID.stack().dropna().apply(
-                lambda ds: liteval(ds)).unstack()))
+            lambda ds: liteval(ds)).unstack()))
     else:
         return df.assign(projectID=df.projectID.apply(lambda x: liteval(x)))
 
@@ -286,7 +272,7 @@ def select_by_projectID(df, projectID, dataset_name=None):
         return df.query("projectID == @projectID")
     else:
         return df[df['projectID'].apply(lambda x:
-                  projectID in sum(x.values(), []))]
+                                        projectID in sum(x.values(), []))]
 
 
 def update_saved_matches_for_(name):
@@ -364,9 +350,9 @@ def parmap(f, arg_list, config=None):
         return list(map(f, arg_list))
 
 
-
 country_map = pd.read_csv(_package_data('country_codes.csv'))\
-                .replace({'name':{'Czechia': 'Czech Republic'}})
+                .replace({'name': {'Czechia': 'Czech Republic'}})
+
 
 def country_alpha2(country):
     """
@@ -382,7 +368,9 @@ def country_alpha2(country):
 
 def convert_alpha2_to_country(df):
     df = get_obj_if_Acc(df)
-    return df.assign(Country = df.Country
+    dic = {'EL': 'GR',  # needed, as some datasets use for Greece and United K.
+           'UK': 'GB'}  # codes that are not conform to ISO 3166-1 alpha2.
+    return df.assign(Country=df.Country.replace(dic)
                      .map(country_map.set_index('alpha_2')['name']))
 
 
@@ -391,8 +379,7 @@ def convert_country_to_alpha2(df):
     alpha2 = df.Country.map(country_map.set_index('name')['alpha_2'])\
                .fillna(country_map.dropna(subset=['official_name'])
                        .set_index('official_name')['alpha_2'])
-    return df.assign(Country = alpha2)
-
+    return df.assign(Country=alpha2)
 
 
 def breakdown_matches(df):
@@ -413,12 +400,12 @@ def breakdown_matches(df):
     if isinstance(df.projectID.iloc[0], list):
         sources = [df.powerplant.get_name()]
         single_source_b = True
-    else :
-        sources = set(df.projectID.apply(dict.keys).apply(list).sum())
+    else:
+        sources = df.projectID.apply(list).explode().unique()
         single_source_b = False
     sources = pd.concat(
-            [getattr(data, s)().set_index('projectID')
-             for s in sources], sort=False)
+        [getattr(data, s)().set_index('projectID')
+         for s in sources], sort=False)
     if df.index.nlevels > 1:
         stackedIDs = (df['projectID'].stack()
                       .apply(pd.Series).stack()
@@ -435,7 +422,62 @@ def breakdown_matches(df):
             .reindex(stackedIDs)
             .set_axis(stackedIDs.to_frame('projectID')
                       .set_index('projectID', append=True).droplevel(-2).index,
-                      inplace=False))
+                      inplace=False)
+            .rename_axis(index=['id', 'source', 'projectID']))
+
+
+def restore_blocks(df, mode=2, config=None):
+    """
+    Restore blocks of powerplants from a matched dataframe.
+
+    This function breaks down all matches. For each match separately it selects
+    blocks from only one input data source.
+    For this selection the following modi are available:
+
+        1. Select the source with most number of blocks in the match
+
+        2. Select the source with the highest reliability score
+
+    Parameters
+    ----------
+    df : pd.DataFrame
+        Matched data with not empty projectID-column. Keys of projectID must
+        be specified in powerplantmatching.data.data_config
+    """
+    from .data import OPSD
+    df = get_obj_if_Acc(df)
+    assert('projectID' in df)
+
+    config = get_config() if config is None else config
+
+    bd = breakdown_matches(df)
+    if mode == 1:
+        block_map = (bd.reset_index(['source'])['source'].groupby(level='id')
+                     .agg(lambda x: pd.Series(x).mode()[0]))
+        blocks_i = pd.MultiIndex.from_frame(block_map.reset_index())
+        res = bd.reset_index('projectID').loc[blocks_i].set_index('projectID',
+                                                                  append=True)
+    elif mode == 2:
+        sources = df.projectID.apply(list).explode().unique()
+        rel_scores = pd.Series({s: config[s]['reliability_score']
+                                for s in sources})\
+                       .sort_values(ascending=False)
+        res = pd.DataFrame().rename_axis(index='id')
+        for s in rel_scores.index:
+            subset = bd.reindex(index=[s], level='source')
+            subset_i = subset.index.unique(
+                'id').difference(res.index.unique('id'))
+            res = pd.concat([res, subset.reindex(index=subset_i, level='id')])
+    else:
+        raise ValueError(f'Given `mode` must be either 1 or 2 but is: {mode}')
+
+    res = res.sort_index(level='id').reset_index(level=[0, 1])
+
+    # Now append Block information from OPSD German list:
+    df_blocks = (OPSD(rawDE_withBlocks=True)
+                 .rename(columns={'name_bnetza': 'Name'}))['Name']
+    res.update(df_blocks)
+    return res
 
 
 def parse_Geoposition(location, zipcode='', country='',
@@ -482,6 +524,7 @@ def parse_Geoposition(location, zipcode='', country='',
         return pd.Series({'Name': location, 'Country': country,
                           'lat': gdata.latitude, 'lon': gdata.longitude})
 
+
 def fill_geoposition(df, use_saved_locations=False, saved_only=False):
     """
     Fill missing 'lat' and 'lon' values. Uses geoparsing with the value given
@@ -504,17 +547,19 @@ def fill_geoposition(df, use_saved_locations=False, saved_only=False):
                        'want to enable it.')
 
     if use_saved_locations:
-        locs = pd.read_csv(_package_data('parsed_locations.csv'), index_col=[0, 1])
+        locs = pd.read_csv(_package_data(
+            'parsed_locations.csv'), index_col=[0, 1])
         df = df.where(df[['lat', 'lon']].notnull().all(1),
-                 df.drop(columns=['lat', 'lon'])
-                   .join(locs, on=['Name', 'Country']))
-    if saved_only: return df
+                      df.drop(columns=['lat', 'lon'])
+                      .join(locs, on=['Name', 'Country']))
+    if saved_only:
+        return df
 
     logger.info("Parse geopositions for missing lat/lon values")
     missing = df.lat.isnull()
     geodata = df[missing].apply(
-                lambda ds: parse_Geoposition(ds['Name'], country=ds['Country']),
-                axis=1)
+        lambda ds: parse_Geoposition(ds['Name'], country=ds['Country']),
+        axis=1)
     geodata.drop_duplicates(subset=['Name', 'Country'])\
            .set_index(['Name', 'Country'])\
            .to_csv(_package_data('parsed_locations.csv'), mode='a', header=False)
